#! /usr/bin/env python

# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Script for generating Sphinx-friendly help for the fastimport commands."""


from bzrlib import commands, option, plugin

# See also links which don't become local documents
SEE_ALSO_PLAIN_LINKS = ['formats']

def build_help_topics_for_plugin(plugin_name="fastimport"):
    # TODO: look these up instead of hard-coding them
    cmds = [
        'baz-import',
        'baz-import-branch',
        'fast-export',
        'fast-export-from-cvs',
        'fast-export-from-darcs',
        'fast-export-from-git',
        'fast-export-from-hg',
        'fast-export-from-mnt',
        'fast-export-from-p4',
        'fast-export-from-svn',
        'fast-import',
        'fast-import-filter',
        'fast-import-info',
        'svn-import',
        ]
    for cmd_name in cmds:
        cmd_object = commands.get_cmd_object(cmd_name)
        filename = "%s.txt" % cmd_name
        print "creating %s ..." % (filename,)
        f = open(filename, 'w')
        f.write(get_help_text_for_sphinx(cmd_object))
        f.close


def get_help_text_for_sphinx(cmd, level1='-', level2='~'):
    result = format_section(cmd.name(), '', level1)
    doc = cmd.help()
    if doc is None:
        return result

    # Extract the summary (purpose) and sections out from the text.
    # If a custom usage section was provided, use it instead.
    purpose, sections, order = cmd._get_help_parts(doc)
    if sections.has_key('Usage'):
        usage = sections.pop('Usage')
    else:
        usage = cmd._usage()
    result += format_section("Purpose", purpose, level2)
    result += format_section("Usage", usage, level2)

    # Add the options
    #
    # XXX: optparse implicitly rewraps the help, and not always perfectly,
    # so we get <https://bugs.launchpad.net/bzr/+bug/249908>.  -- mbp
    # 20090319
    options = option.get_optparser(cmd.options()).format_option_help()
    if options.startswith('Options:'):
        options_text = options[len('Options:'):]
    elif options.startswith('options:'):
        # Python 2.4 version of optparse
        options_text = options[len('options:'):]
    else:
        # When does this case happen?
        options_text = options
    result += format_section("Options", deindent(options_text), level2)

    # Add the description
    if sections.has_key(None):
        text = sections.pop(None)
        text = '\n'.join(text.splitlines())
        result += format_section('Description', text, level2)

    # Add the custom sections (e.g. Examples).
    # XXX: These are indented already in the source by a space or two
    # so we may to deindent them (after checking that) in the future.
    if sections:
        for label in order:
            if sections.has_key(label):
                text = sections[label]
                result += format_section(label, deindent(text), level2)
        result += '\n'

    # Add the aliases, source (plug-in) and see also links, if any
    if cmd.aliases:
        result += format_section('Aliases', ', '.join(cmd.aliases), level2)
    plugin_name = cmd.plugin_name()
    if plugin_name is not None:
        result += format_section('From', 'plugin "%s"' % plugin_name, level2)
    see_also = cmd.get_see_also()
    if see_also:
        see_also_links = []
        for topic in see_also:
            if topic in SEE_ALSO_PLAIN_LINKS:
                see_also_links.append(topic)
            else:
                see_also_links.append(":doc:`%s`" % (topic,))
        result += format_section('See also', ', '.join(see_also_links), level2)
    return result


def format_section(hdg, text, hdg_underline):
    """Format a section given a heading and text content."""
    result = "%s\n%s\n\n" % (hdg, hdg_underline * len(hdg))
    if text:
        result += "%s\n\n" % (text,)
    return result


def deindent(text):
    """Remove common leading spaces off lines in text."""
    lines = text.split("\n")
    indents = []
    for line in lines:
        if len(line) > 0:
            indents.append(len(line) - len(line.lstrip(' ')))
    indent = min(indents)
    if indent:
        lines = [l[indent:] for l in lines]
    return '\n'.join(lines)


if __name__ == '__main__':
    #if len(sys.argv) == 0:
    #    print "usage: build-topics-for-plugin plugin-name"
    #    sys.exit(1)
    plugin.load_plugins()
    commands.install_bzr_command_hooks()
    build_help_topics_for_plugin()
